package com.fzm.portal.aop;

import com.tencentcloudapi.common.Credential;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import com.tencentcloudapi.common.profile.ClientProfile;
import com.tencentcloudapi.common.profile.HttpProfile;
import com.tencentcloudapi.faceid.v20180301.FaceidClient;
import com.tencentcloudapi.faceid.v20180301.models.IdCardOCRVerificationRequest;
import com.tencentcloudapi.faceid.v20180301.models.IdCardOCRVerificationResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.After;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author tangtuo
 * @date 2021/9/10 14:27
 */
@Slf4j
@Order(1)
@Component
@Aspect
public class LogAop {

    @Resource
    private HttpServletRequest request;

    private static ThreadLocal<Long> threadLocal = new ThreadLocal<>();

    @Pointcut(value = "execution(public * com.fzm.portal.controller.*.*(..))")
    public void pointCut() {
    }


    /**
     * 前置通知
     *
     * @param joinPoint
     */
    @Before("pointCut()")
    public void deBefore(JoinPoint joinPoint) {
        // 记录此次请求的开始时间
        long start = System.currentTimeMillis();
        threadLocal.set(start);
        // 获取请求路径
        String requestURI = request.getRequestURI();
        // 获取请求参数
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();
        List<Object> parameter = getParameter(method, joinPoint.getArgs());
        log.info("请求路径: {}，请求参数：{}", requestURI, parameter);
    }


    /**
     * 后置通知
     *
     * @param joinPoint
     */
    @After("pointCut()")
    public void doAfter(JoinPoint joinPoint) {
        long end = System.currentTimeMillis();
        Long start = threadLocal.get();
        long l = end - start;
        String requestURI = request.getRequestURI();
        log.info("请求路径：{}，接口耗时：{}ms", requestURI, l);
    }

    /**
     * 根据方法和传入的参数获取请求参数
     */
    private List<Object> getParameter(Method method, Object[] args) {
        List<Object> argList = new ArrayList<>();
        Parameter[] parameters = method.getParameters();
        for (int i = 0; i < parameters.length; i++) {
            //将RequestBody注解修饰的参数作为请求参数
            RequestBody requestBody = parameters[i].getAnnotation(RequestBody.class);
            if (requestBody != null) {
                argList.add(args[i]);
            }
            ModelAttribute modelAttribute = parameters[i].getAnnotation(ModelAttribute.class);
            if (modelAttribute != null) {
                argList.add(args[i]);
            }
            //将RequestParam注解修饰的参数作为请求参数
            RequestParam requestParam = parameters[i].getAnnotation(RequestParam.class);
            if (requestParam != null) {
                Map<String, Object> map = new HashMap<>();
                String key = parameters[i].getName();
                if (!StringUtils.isEmpty(requestParam.value())) {
                    key = requestParam.value();
                }
                map.put(key, args[i]);
                argList.add(map);
            }
            // PathVariable修饰的请求参数
            PathVariable pathVariable = parameters[i].getAnnotation(PathVariable.class);
            if (pathVariable != null) {
                Map<String, Object> map = new HashMap<>();
                String key = parameters[i].getName();
                if (StringUtils.isNotBlank(pathVariable.value())) {
                    key = pathVariable.value();
                }
                map.put(key, args[i]);
                argList.add(map);
            }
        }
        return argList;
    }



        public static void main(String[]args){
            try {
                // 实例化一个认证对象，入参需要传入腾讯云账户secretId，secretKey,此处还需注意密钥对的保密
                // 密钥可前往https://console.cloud.tencent.com/cam/capi网站进行获取
                Credential cred = new Credential("AKIDlMm3EC2dxXw50f4C4f092ey2pLaaKnyt", "g5D4ey1nJZXEhtoSovkIZYGhqjLgkgNo");
                // 实例化一个http选项，可选的，没有特殊需求可以跳过
                HttpProfile httpProfile = new HttpProfile();
                httpProfile.setEndpoint("faceid.tencentcloudapi.com");
                // 实例化一个client选项，可选的，没有特殊需求可以跳过
                ClientProfile clientProfile = new ClientProfile();
                clientProfile.setHttpProfile(httpProfile);
                // 实例化要请求产品的client对象,clientProfile是可选的
                FaceidClient client = new FaceidClient(cred, "ap-shanghai", clientProfile);
                // 实例化一个请求对象,每个接口都会对应一个request对象
                IdCardOCRVerificationRequest req = new IdCardOCRVerificationRequest();
                String url = "https://test-nft.oss-cn-hangzhou.aliyuncs.com/20210913/ff8e541497d1482895be1f038bb0430e/微信图片_20210913144401.jpg";
                req.setImageUrl(url);
                // 返回的resp是一个IdCardOCRVerificationResponse的实例，与请求对象对应
                IdCardOCRVerificationResponse resp = client.IdCardOCRVerification(req);
                // 输出json格式的字符串回包
                System.out.println(IdCardOCRVerificationResponse.toJsonString(resp));
            } catch (TencentCloudSDKException e) {
                System.out.println(e.toString());
            }
        }

}
